﻿
-- データベース ユーザーの作成
CREATE USER Manager WITH PASSWORD = 'P@ssword'
CREATE USER User1 WITH PASSWORD = 'P@ssword'
CREATE USER User2 WITH PASSWORD = 'P@ssword'

-- テーブルの作成。colUserName 列にアクセス可能なユーザーの名前を格納するようにする
CREATE TABLE rsTest1
( colUserName sysname
 ,colA int
 ,colB varchar(200) )

-- データを 4件追加
INSERT rsTest1
 VALUES ('User1', 1, 'AAA') 
       ,('User1', 2, 'BBB') 
       ,('User2', 3, 'CCC') 
       ,('User2', 4, 'DDD') 

-- データの確認
SELECT * FROM rsTest1

-- データベース ユーザーに、テーブルに対する SELECT 権限を付与
GRANT SELECT ON rsTest1 TO Manager
GRANT SELECT ON rsTest1 TO User1
GRANT SELECT ON rsTest1 TO User2
go

-- ユーザー定義関数の作成
-- ユーザーはユーザーのデータ、Manager は全データを制御
CREATE FUNCTION rsFunc1(@UserName AS sysname)
RETURNS TABLE
WITH SCHEMABINDING
AS
 RETURN
   SELECT 1 AS result
    WHERE USER_NAME() = @UserName
       OR USER_NAME() = 'Manager'
go

-- フィルターの作成
-- 上で作成したユーザー定義関数に、該当テーブルの colUserName 列を与える
CREATE SECURITY POLICY rsFilter1
 ADD FILTER PREDICATE dbo.rsFunc1(colUserName) 
  ON dbo.rsTest1
 WITH (STATE = ON)
go

-- 何も権限が与えられていないユーザー
SELECT * FROM rsTest1


-- Manager をシミュレート
EXECUTE AS USER = 'Manager'
SELECT * FROM rsTest1
REVERT


-- User1 をシミュレート
EXECUTE AS USER = 'User1'
SELECT * FROM rsTest1
REVERT


-- User2 をシミュレート
EXECUTE AS USER = 'User2'
SELECT * FROM rsTest1
REVERT


-- 行レベル セキュリティを無効化する（フィルターを無効化）
ALTER SECURITY POLICY rsFilter1
 WITH (STATE = OFF)






