﻿
-- CPU 利用の多いクエリ TOP 10
SELECT TOP 10
    total_worker_time  / execution_count / 1000.0 AS [平均 CPU 時間（ミリ秒）]
  , total_elapsed_time / execution_count / 1000.0 AS [平均実行時間（ミリ秒）]
  , SUBSTRING(text, (statement_start_offset / 2) + 1, 
        ((CASE statement_end_offset
          WHEN -1 THEN DATALENGTH(text)
          ELSE statement_end_offset
          END - statement_start_offset) / 2) + 1) AS sql
FROM
 sys.dm_exec_query_stats 
  CROSS APPLY
   sys.dm_exec_sql_text(sql_handle)
ORDER BY 1 DESC


-- インデックス サイズの確認
SELECT OBJECT_NAME(ps.object_id) AS objName
      ,ps.index_id, MAX(i.name) AS idxName
      ,SUM(ps.used_page_count) * 8 / 1024.0 AS sizeMB
      ,SUM(ps.row_count) AS rowCnt
FROM
 sys.dm_db_partition_stats ps
     LEFT OUTER JOIN sys.indexes i
        ON ps.object_id = i.object_id
       AND ps.index_id = i.index_id
WHERE OBJECTPROPERTYEX(ps.object_id, 'IsUserTable') = 1
GROUP BY ps.object_id, ps.index_id


-- t1 テーブルの断片化状況を確認
SELECT OBJECT_NAME(object_id), index_id, avg_fragmentation_in_percent
      ,index_level, page_count, record_count
FROM
 sys.dm_db_index_physical_stats(DB_ID(), OBJECT_ID('t1'), NULL, NULL, 'DETAILED')
ORDER BY index_id, index_level DESC

-- 未使用インデックスをリストアップ
SELECT o.name AS table_name, i.name AS index_name, *
 FROM sys.objects o
      INNER JOIN sys.indexes i ON o.object_id = i.object_id
 WHERE o.type = 'U'
   AND i.index_id
       NOT IN
         ( SELECT s.index_id 
            FROM sys.dm_db_index_usage_stats s 
             WHERE s.object_id = i.object_id
               AND i.index_id = s.index_id
               AND database_id = DB_ID() )

-- Missing インデックス（不足しているインデックス）
SELECT group_handle, i.*, gs.*
FROM sys.dm_db_missing_index_group_stats gs
      INNER JOIN sys.dm_db_missing_index_groups g
              ON gs.group_handle = g.index_group_handle
      INNER JOIN sys.dm_db_missing_index_details i
              ON i.index_handle = g.index_handle

-- プロシージャ キャッシュの参照
SELECT query_plan, size_in_bytes / 8192 AS [使用ページ数], *
FROM
  sys.dm_exec_cached_plans 
    CROSS APPLY
      sys.dm_exec_query_plan (plan_handle)




-- ロック、ブロッキングに関する情報
SELECT * FROM sys.dm_tran_locks

EXEC sp_who2

/*
sp_lock は未サポート
*/

-- 現在の待機に関する情報。S2 以上だと確認可能
SELECT * FROM sys.dm_os_waiting_tasks




-- Wait Stats（待機に関する情報の取得）. S2 以上だと確認可能
SELECT * FROM sys.dm_os_wait_stats
 ORDER BY wait_time_ms DESC


-- パフォーマンス カウンター情報の取得. S2 以上だと確認可能
SELECT * FROM sys.dm_os_performance_counters


-- オブジェクトごとのメモリ使用量の内訳. S2 以上だと確認可能
SELECT 
  DB_NAME (b.database_id), OBJECT_NAME(p.object_id), p.index_id
 ,COUNT(*) As 使用ページ数
FROM
 sys.allocation_units a
  INNER JOIN  sys.partitions p ON a.container_id = p.hobt_id
  INNER JOIN  sys.dm_os_buffer_descriptors b
      ON a.allocation_unit_id = b.allocation_unit_id
WHERE b.database_id = DB_ID()
GROUP BY b.database_id, p.object_id, p.index_id
ORDER BY 使用ページ数 DESC

